"""
Contains a class that handles endpoint configuration. Its main
functions are reading the configuration files and restoring the
defaults if some keys are absent there.
"""
from .enums import *
import json
import copy
import re
import os


class ResourceConfig:
    """
    Properties of this class correspond to the keys in a configuration file
    for one of the resources this endpoint communicates with.
    """

    rxExt = re.compile('\\.[^.]*$')

    def __init__(self, fnameConfig=None):
        self.platform = 'tsakorpus'
        self.transport_protocol = 'https'
        self.host = '127.0.0.1'
        self.port = '5000'
        self.url_path = '127.0.0.1'
        self.resource_base_url = 'http://127.0.0.1'
        self.titles = []
        self.descriptions = []
        self.authors = []
        self.contacts = []
        self.extents = []
        self.history = []
        self.restrictions = []
        self.max_hits = 10
        self.basic_search_capability = True
        self.advanced_search_capability = False
        self.hits_supported = True
        self.adv_supported = False
        self.supported_layers = []
        self.resources = []
        self.search_lang_id = ''

        self.query_timeout = 60

        self.boolParams = set(k for k in self.__dict__
                              if type(self.__dict__[k]) == bool)
        self.intParams = set(k for k in self.__dict__
                             if type(self.__dict__[k]) == int)
        self.lsParams = {}

        # dictionaries where values are strings
        self.dict_sParams = {}

        # dictionaries where values are lists of strings
        self.dict_lsParams = {}

        # dictionaries where values are dictionaries {k: string}
        self.dict_dParams = {}

        if fnameConfig is not None and os.path.exists(fnameConfig):
            self.load_settings(fnameConfig)


    def load_settings(self, fnameConfig):
        """
        Load configuration for one of the resources from a JSON file.
        """
        with open(fnameConfig, 'r', encoding='utf-8') as fConfig:
            config = json.load(fConfig)
        for k, v in config.items():
            setattr(self, k, v)

    def as_dict(self):
        """
        Return current settings as a dictionary.
        """
        dictSettings = copy.deepcopy(vars(self))
        for k in [_ for _ in dictSettings.keys()]:
            if dictSettings[k] is None:
                dictSettings[k] = ''
        return dictSettings

    def gui_str_to_dict(self, s, value_type='list'):
        """
        Process one input string that describes a dictionary.
        """
        d = {}
        s = s.replace('\r', '').strip()
        s = re.sub('\n\n+', '\n', s, flags=re.DOTALL)
        if value_type == 'dict':
            prevKey = ''
            curData = {}
            for line in s.split('\n'):
                if not line.startswith(' '):
                    curKey = line.strip(': ')
                    if len(prevKey) > 0 and curKey != prevKey:
                        d[prevKey] = curData
                        curData = {}
                    prevKey = curKey
                else:
                    line = line.strip()
                    if ':' not in line:
                        continue
                    k, v = line.split(':')
                    k = k.rstrip()
                    v = v.lstrip()
                    curData[k] = v
            if len(curData) > 0:
                d[prevKey] = curData
        else:
            for line in s.split('\n'):
                line = line.strip()
                if ':' not in line:
                    continue
                k, v = line.split(':')
                k = k.rstrip()
                v = v.lstrip()
                if value_type == 'list':
                    if len(v) <= 0:
                        v = []
                    else:
                        v = [vp.strip() for vp in v.split(',')]
                d[k] = v
        return d

    def processed_gui_config(self, data):
        """
        Turn form data filled by the user in the configuration GUI to
        a dictionary in the correct format.
        """
        dictConfig = {}
        for f in self.boolParams:
            if f in data and len(data[f]) > 0:
                dictConfig[f] = True
            else:
                dictConfig[f] = False
        for f in self.intParams:
            if f in data and len(data[f]) > 0:
                dictConfig[f] = int(data[f])
        for f in self.lsParams:
            if f in data and len(data[f]) > 0:
                dictConfig[f] = [v.strip() for v in data[f].replace('\r', '').strip().split('\n')]
            else:
                dictConfig[f] = []
        for f in self.dict_sParams:
            if f in data and len(data[f]) > 0:
                dictConfig[f] = self.gui_str_to_dict(data[f], value_type='string')
            else:
                dictConfig[f] = {}
        for f in self.dict_lsParams:
            if f in data and len(data[f]) > 0:
                dictConfig[f] = self.gui_str_to_dict(data[f], value_type='list')
            else:
                dictConfig[f] = {}
        for k, v in data.items():
            if '%' in k:
                continue
            if k not in dictConfig:
                dictConfig[k] = v
        return dictConfig

    def save_settings(self, fnameOut, data=None):
        """
        Save current or new configuration as a JSON file (can be used to edit
        configuration files through a web interface).
        """
        if data is None or len(data) <= 0:
            dictConfig = self.as_dict()
        else:
            dictConfig = self.processed_gui_config(data)
        with open(fnameOut, 'w', encoding='utf-8') as fOut:
            json.dump(dictConfig, fOut, sort_keys=True, ensure_ascii=False, indent=2)


def read_configs(configDir='./config'):
    """
    Load all configuration files from the configuration directory
    (one file per resource). Initialize ResourceConfig instances for
    each of them. Return a dictionary where the keys are filenames and
    the values contain respective ResourceConfig objects.
    """
    configs = {}
    for fname in os.listdir(configDir):
        if not fname.lower().endswith('.json'):
            continue
        fnameFull = os.path.join(configDir, fname)
        fnameNoExt = ResourceConfig.rxExt.sub('', fname)
        configs[fnameNoExt] = ResourceConfig(fnameFull)
    return configs
